import os
import os.path as op
import datetime
from flask import Flask, flash, request, Response, json, redirect, url_for
from flask_sqlalchemy import SQLAlchemy
from sqlalchemy.event import listens_for
from CypressBoat import app
from wtforms import validators

import flask_admin as fadmin
from flask_admin.contrib import sqla
from flask_admin.contrib.sqla import filters
from flask_admin.contrib.sqla.view import func
from migrate.versioning import api
import flask_admin.model.typefmt as typefmt


from flask_wtf import Form
from flask_admin.contrib.sqla.ajax import QueryAjaxModelLoader
from flask_admin.model.fields import AjaxSelectField, AjaxSelectMultipleField
from wtforms import StringField, RadioField, BooleanField, TextAreaField 
from wtforms.validators import DataRequired, Length, InputRequired
from flask_admin import BaseView, expose



from flask_security import Security, current_user, SQLAlchemyUserDatastore, \
    UserMixin, RoleMixin, login_required, user_registered


def GetCurrentUser():
    ''' not sure why i need to do this, isnt current_user the same? '''
    return User.query.get(current_user.id)

# setup main database link
db = SQLAlchemy()

# Define m2m table linking users to roles
roles_users = db.Table('roles_users',
        db.Column('user_id', db.Integer(), db.ForeignKey('user.id')),
        db.Column('role_id', db.Integer(), db.ForeignKey('role.id')))

class Role(db.Model, RoleMixin):
    id = db.Column(db.Integer(), primary_key=True)
    name = db.Column(db.String(80), unique=True)
    description = db.Column(db.String(255))
    def __str__(self):
        return self.name

class User(db.Model, UserMixin):
    id = db.Column(db.Integer, primary_key=True)
    first_name = db.Column(db.String(100))
    last_name = db.Column(db.String(100))
    username = db.Column(db.String(80), unique=True)
    email = db.Column(db.String(255), unique=True)
    password = db.Column(db.String(255))
    about_me = db.Column(db.String(1024))
    active = db.Column(db.Boolean())
    confirmed_at = db.Column(db.DateTime())
    last_login_at= db.Column(db.DateTime())
    current_login_at= db.Column(db.DateTime())
    last_login_ip= db.Column(db.String(255))
    current_login_ip= db.Column(db.String(255))
    login_count = db.Column(db.Integer)
    upload_quota = db.Column(db.Integer) # max amount this user can upload (bytes)
    upload_quota_used = db.Column(db.Integer) # amount this user has uploaded (bytes)
    profile_image_id = db.Column(db.Integer) # Image ID for this users profile ID, or None
    roles = db.relationship('Role', secondary=roles_users,
                            backref=db.backref('users', lazy='dynamic'))
    def has_role(self, role):
        return role in self.roles
    def __str__(self):
        return str(self.username)


# Setup Flask-Security
user_datastore = SQLAlchemyUserDatastore(db, User, Role)

@user_registered.connect_via(app)
def user_registered_sighandler(app, user, confirm_token):
    ''' Anyone who signs up currently gets automatic author status '''
    default_role = user_datastore.find_role("author")
    user.upload_quota = app.config("USER_DEFAULT_UPLOAD_QUOTA")
    user.upload_quota_used = 0
    user_datastore.add_role_to_user(user, default_role)
    db.session.commit()

from flask_login import user_logged_in, user_logged_out

@user_logged_in.connect_via(app)
def on_user_logged_in(sender, user):
    return
    #if user.username:
    #    flash("Welcome " + str(user.username) + "!")

@user_logged_out.connect_via(app)
def on_user_logged_out(sender, user):
    return
    #if user.username:
    #    flash("Goodbye " + str(user.username) + "!")


# many-to-many table linking products and the detailed tags for user searching
product_tags_table = db.Table('product_tags', db.Model.metadata,
                         db.Column('product_id', db.Integer, db.ForeignKey('product.id')),
                         db.Column('tag_id', db.Integer, db.ForeignKey('tag.id'))
                         )

# many-to-many table linking products to images
product_images_table = db.Table('product_images', db.Model.metadata,
                         db.Column('product_id', db.Integer, db.ForeignKey('product.id')),
                         db.Column('image_id', db.Integer, db.ForeignKey('image.id'))
                         )


class Image(db.Model):
    ''' An uploaded image. '''
    title = db.Column(db.String(120), unique=True) # unique file identifier (used with flask_uploads)    
    id = db.Column(db.Integer, primary_key=True)    
    date = db.Column(db.DateTime) # date at which the content was posted    
    meta_data = db.Column(db.String(120)) # any descriptive info about the file
    user = db.relationship(User, backref='images') # user who posted the content
    user_id = db.Column(db.Integer(), db.ForeignKey(User.id)) # user id who posted the content
    def __str__(self):
        return self.title


class Product(db.Model):
    ''' Item for sale. '''
    id = db.Column(db.Integer, primary_key=True)
    title = db.Column(db.String(120), unique=True) # main title
    date = db.Column(db.DateTime) # date at which the content was last modified
    summary = db.Column(db.Text) # short summary
    price = db.Column(db.String(120)) # price string
    text = db.Column(db.Text) # main transcript / description
    user = db.relationship(User, backref='products') # user who posted the content
    user_id = db.Column(db.Integer(), db.ForeignKey(User.id)) # user id who posted the content
    tags = db.relationship("Tag", secondary=product_tags_table) # detailed product tags
    summary_image = db.relationship("Image") # primary image used for product summaries             
    summary_image_id = db.Column(db.Integer(), db.ForeignKey(Image.id)) # primary image used for product summaries     
    images = db.relationship("Image", secondary=product_images_table) # images for this post
    comments = db.relationship("ProductComment", backref='product', lazy='dynamic')

    def get_comments(self):
        return Comment.query.filter_by(product_id=product.id).order_by(Comment.timestamp.desc())
    def __str__(self):
        return self.title

    def GetTagStringList(self):
        ''' returns a list of tags associated with the product, in string format '''
        return([str(t) for t in self.tags])
    
class ProductComment(db.Model):
    ''' This is a comment on a Post '''
    id = db.Column(db.Integer, primary_key=True)
    title = db.Column(db.String(140), unique=True)   
    body = db.Column(db.String(140))
    timestamp = db.Column(db.DateTime)
    #product = db.relationship("Product", backref='comments') # user who posted the content
    product_id = db.Column(db.Integer, db.ForeignKey('product.id'))
    user = db.relationship(User, backref='product_comments') # user who posted the content
    user_id = db.Column(db.Integer(), db.ForeignKey(User.id)) # user id who posted the content
    def __repr__(self):
        return '<ProductComment %r>' % (self.body)

class Tag(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.Unicode(64))
    def __str__(self):
        return self.name

class UserInfo(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    key = db.Column(db.String(64), nullable=False)
    value = db.Column(db.String(64))
    user_id = db.Column(db.Integer(), db.ForeignKey(User.id))
    user = db.relationship(User, backref='info')
    def __str__(self):
        return '%s - %s' % (self.key, self.value)

# Customized User model admin
class UserAdmin(sqla.ModelView):
    inline_models = (UserInfo,)
    def is_accessible(self):
        return current_user.has_role('admin')

class TagsAdmin(sqla.ModelView):
    def __init__(self, session):
        # Just call parent class with predefined model.
        super(TagsAdmin, self).__init__(Tag, session)
    def is_accessible(self):
        return current_user.has_role('admin')

# Customized Post model admin
class ProductAdmin(sqla.ModelView):

    roles_accepted=['admin', 'editor']

    # Visible columns in the list view
    column_exclude_list = ['text', 'user']

    # List of columns that can be sorted. For 'user' column, use User.username as
    # a column.
    column_sortable_list = ('title', ('user', 'user.username'), 'date')

    # Rename 'title' columns to 'Post Title' in list view
    column_labels = dict(title='Title')

    column_searchable_list = ('title', User.username, 'tags.name')

    column_filters = ('date',
                      'tags')

    def date_format(view, value):
        return value.strftime('%Y-%m-%d')

    MY_DEFAULT_FORMATTERS = dict(typefmt.BASE_FORMATTERS)
    MY_DEFAULT_FORMATTERS.update({
        datetime.datetime: date_format
    })

    form_args = dict(
        time = dict(validators=[DataRequired()],format='%Y-%m-%d %H:%M')
    )
    form_widget_args = dict(
        time={'data-date-format': u'YYYY-MM-DD HH:mm'} 
    )
    column_type_formatters = MY_DEFAULT_FORMATTERS

    # Pass arguments to WTForms. In this case, change label for text field to
    # be 'Big Text' and add required() validator.
    form_args = dict(
                    title=dict(label='Title', validators=[validators.required()])
                )

    form_ajax_refs = {
        'user': {
            'fields': (User.username, User.email)
        },
        'tags': {
            'fields': (Tag.name,)
        }
    }

    def is_accessible(self):
        if not current_user.is_authenticated: return False
        if current_user.has_role('admin') or current_user.has_role('editor'):
            return True
        else: return False

    #---------------------------------------------------
    # user version automatically sets certain fields

    form_excluded_columns = ('user', 'date')

    # override the model creator to add a user
    def create_model(self, form):
        model = super().create_model(form)
        model.user = current_user
        model.user_id = current_user.id
        model.date = datetime.datetime.now()
        return model

    def on_model_change(self, form, model, is_created):
        model.user = current_user
        model.user_id = current_user.id
        model.date = datetime.datetime.now()


# the following section of imports and code is used to support the search function which uses the same tag entry system as the admin forms, even though it is technically seperate from the admin functionality


def get_loader_by_name(name):
    _dicts = {
        'tag': QueryAjaxModelLoader(
            'tag',
            db.session, Tag,
            fields=['name'],
            page_size=10,
            placeholder="Select tags"
        )
    }
    return _dicts.get(name, None)


class SearchForm(Form):
    item_type = RadioField('Type', choices=[('product','Product')])
    #required_tags = StringField('required_tags', validators=[DataRequired()])
    #optional_tags = StringField('optional_tags', validators=[DataRequired()])
    required_tags = AjaxSelectMultipleField(
        loader=get_loader_by_name('tag')
    )
    optional_tags = AjaxSelectMultipleField(
        loader=get_loader_by_name('tag')
    )


class SearchView(BaseView):

    def __init__(self, name=None, category=None,
                 endpoint=None, url=None,
                 template='admin/index.html',
                 menu_class_name=None,
                 menu_icon_type=None,
                 menu_icon_value=None):
        super().__init__(name or babel.lazy_gettext('Home'),
                                             category,
                                             endpoint or 'admin',
                                             url or '/admin',
                                             'static',
                                             menu_class_name=menu_class_name,
                                             menu_icon_type=menu_icon_type,
                                             menu_icon_value=menu_icon_value)
        self._template = template

    @expose('/', methods=('GET',))
    def index_view(self):
        _form = SearchForm()
        if _form.validate_on_submit():
            flash('Your comment has been posted successfully.')
        return self.render(self._template, form=_form)

    @expose('/', methods=('POST',))
    def post_view(self):
        form = SearchForm()
        #if _form.validate_on_submit():
        item_type = "product"#form.item_type.data
        if len(form.required_tags.data) > 0:
            required_tags = ",".join([d.name for d in form.required_tags.data])
        else: required_tags = None
        if len(form.optional_tags.data) > 0:
            optional_tags = ",".join([d.name for d in form.optional_tags.data])
        else: optional_tags = None

        if item_type == None:
            flash('You have to select one type:' + res)
            return self.render(self._template, form=form)
        if required_tags or optional_tags: QueryString = "?"
        else: QueryString = ""
        if required_tags: QueryString = QueryString + "requiredtags=" + required_tags
        if required_tags and optional_tags: QueryString = QueryString + "&"
        if optional_tags: QueryString = QueryString + "optionaltags=" + optional_tags
        if item_type == "product":
            return redirect('/products'+QueryString)
        return self.render(self._template, form=form)

    @expose('/ajax/lookup/')
    def ajax_lookup(self):
        name = request.args.get('name')
        query = request.args.get('query')
        offset = request.args.get('offset', type=int)
        limit = request.args.get('limit', 10, type=int)
        loader = get_loader_by_name(name)
        if not loader: abort(404)
        data = [loader.format(m) for m in loader.get_list(query, offset, limit)]
        return Response(json.dumps(data), mimetype='application/json')



#-----------------------------------------------------------------------------------------------------------------------------
# image uploading etc.
from jinja2 import Markup
from flask_admin.model import template

#-------------------------------------------------------------------------------------
# File upload /deletion

import uuid
from PIL import Image as PILImage

def unique_id():
    return hex(uuid.uuid4().time)[2:-1]

def SafeDeleteFile(fname):
    ''' returns the number of bytes removed from the system '''
    try:
        bytesr = os.path.getsize(fname)
        os.remove(fname)
        return bytesr
    except OSError:
        return 0


# override the normal image upload field behaviour so it doesn't save the image,
# we will handle that ourselves
class MyImageUploadField(fadmin.form.ImageUploadField):
    def _save_file(self, data, filename):
        pass

class ImageAdmin(sqla.ModelView):
    
    #---------------------------------------------------
    # user version automatically sets certain fields

    form_excluded_columns = ('user', 'date', 'id')

    roles_accepted=['admin', 'editor']

    def _list_thumbnail(view, context, model, name):
        fname = app.config['UPLOADED_IMAGE_URL'] + "thumbnails/"  + str(model.id) +".jpg"
        return Markup('<img src="%s">' % (fname))

    column_formatters = {'preview': _list_thumbnail}

    # custom formatting for the image display
    list_template = 'admin/List_ImageAdmin.html'
    list_formatters = dict(value=template.macro('color'))
    
    # Visible columns in the list view
    form_extra_fields = {'preview': MyImageUploadField('Image')}
    column_list = ('title', 'id', 'date', 'preview')
    column_exclude_list = ['meta_data', 'user', 'id']
    column_labels = dict(title='Title', date='Created', preview='Preview')
    form_excluded_columns = ('file_id', 'user', 'date')
    column_searchable_list = ['title']
    column_filters = ('date', 'title')

    def date_format(view, value):
        return value.strftime('%Y-%m-%d')

    MY_DEFAULT_FORMATTERS = dict(typefmt.BASE_FORMATTERS)
    MY_DEFAULT_FORMATTERS.update({
        datetime.datetime: date_format
    })
    
    # Pass arguments to WTForms. In this case, change label for text field to
    # be 'Big Text' and add required() validator.
    form_args = dict(
                    title=dict(label='Title', validators=[validators.required()]),
                    preview=dict(label='Image File', validators=[validators.required()])
                )

    form_widget_args = dict(
        time={'data-date-format': u'YYYY-MM-DD HH:mm'} 
    )
    column_type_formatters = MY_DEFAULT_FORMATTERS

    def is_accessible(self):
        if not current_user.is_authenticated: return False
        if current_user.has_role('author'):
            return True
        else: return False

    # if not admin or editor, only show the items owned by the current user
    def get_query(self):
        return Image.query.filter(Image.user_id == current_user.id)        

    def get_count_query(self):
        return self.session.query(func.count('*')).filter(Image.user_id == current_user.id)

    # override model deletion
    def delete_model_intern(self, model):
        pre = app.config['UPLOADED_IMAGE_FOLDER']    
        bytesr = SafeDeleteFile(pre + str(model.id) +".jpg")
        bytesr = bytesr + SafeDeleteFile(pre + "thumbnails/"  + str(model.id) +".jpg")
        #flash("deleting image")
        #flash("before used " + str(current_user.upload_quota_used))
        current_user.upload_quota_used = current_user.upload_quota_used - bytesr
        #flash("after deletion you are using" + str(current_user.upload_quota_used))
            
    def delete_model(self, model):
        self.delete_model_intern(model)
        db.session.delete(model)        
        db.session.commit()
        return True

    # override the model creator to add a user
    def create_model(self, form):
        if form.preview.data is None:
            flash("Please choose an image file.")
            return
        model = super().create_model(form)
        model = self.create_model_intern(form, model)
        db.session.commit()
        return model
    
    # called from create_model etc.
    def create_model_intern(self, form, model):
        cu = GetCurrentUser()

        # fix for users that didn't initialise properly
        if cu.upload_quota is None:
            cu.upload_quota = app.config['USER_DEFAULT_UPLOAD_QUOTA']
        if cu.upload_quota_used is None:
            cu.upload_quota_used = 0

        # process the image data to create a fullsize and thumbnail
        image = PILImage.open(form.preview.data)
        thumbnail = image.copy()
        thumbnail.thumbnail((50, 50), PILImage.ANTIALIAS)
        pre = app.config['UPLOADED_IMAGE_FOLDER']

        fname = pre + str(model.id) +".jpg"
        image.convert('RGB').save(fname, "jpeg")
        BytesUsed = os.path.getsize(fname)        
    
        tfname = pre + "thumbnails/"  + str(model.id) +".jpg"
        thumbnail.convert('RGB').save(tfname, "jpeg")
        BytesUsed += os.path.getsize(tfname)

        # check that the user quote was not exceeded
        if cu.upload_quota_used + BytesUsed > cu.upload_quota:
            SafeDeleteFile(fname)
            SafeDeleteFile(tfname)
            db.session.remove(model)
            flash("Your user upload quota has been exceeded, the image was not been uploaded.")
        else:
            # add other attributes to the image model
            #flash("creating new image")
            TotalBytesUsed = cu.upload_quota_used + BytesUsed
            #flash("before used " + str(cu.upload_quota_used))
            cu.upload_quota_used = TotalBytesUsed
            #flash("bytes used " + str(BytesUsed))
            #flash("you have used total " + str(cu.upload_quota_used))
            model.user = cu
            model.user_id = cu.id
            model.date = datetime.datetime.now()
        return model

    def update_model(self, form, model):
        if form.preview.data is None:
            flash("Please choose an image file.")
            return False
        # if the model was created already, delete it first and make a new one        
        self.delete_model_intern(model)
        self.create_model_intern(form, model)
        db.session.commit()
        return True
        

#----------------------------------------------------------------------------------------------------------------------------
# init and config

def build_sample_db():
    """
    Populate a small db with some example entries.
    """
    
    import random

    db.drop_all()
    db.create_all()

    # create roles
    admin_role = user_datastore.create_role(name="admin", description="Full management rights")
    editor_role = user_datastore.create_role(name="editor", description="Full content management rights")
    author_role = user_datastore.create_role(name="author", description="Self publishing & editing rights")

    # create initial admin user
    admin_user = user_datastore.create_user(first_name="Christian", last_name="Lynch", username="azorin", email='cnslynch@gmail.com', password='liquido3', confirmed_at=datetime.datetime.now())
    user_datastore.add_role_to_user(admin_user, admin_role)
    user_datastore.add_role_to_user(admin_user, editor_role)
    user_datastore.add_role_to_user(admin_user, author_role)
    christian_user = user_datastore.create_user(first_name="Christian", last_name="Lynch", username="christian", email='cnslynch@outlook.com', password='liquido3', confirmed_at=datetime.datetime.now())
    user_datastore.add_role_to_user(christian_user, author_role)
    other_user = user_datastore.create_user(first_name="Xiaoxiao", last_name="Yan", username="xiaoxiao", email='info@cypressBoat.co.uk', password='goodlucktoyoudarling5888', confirmed_at=datetime.datetime.now())
    user_datastore.add_role_to_user(other_user, author_role)
    user_datastore.add_role_to_user(other_user, editor_role)
    user_datastore.add_role_to_user(other_user, admin_role)

    '''
    user_list = []
    for i in range(len(first_names)):
        user = User()
        user.first_name = first_names[i]
        user.username = first_names[i].lower()
        user.last_name = last_names[i]
        user.email = user.username + "@example.com"
        user_list.append(user)
        db.session.add(user)
    '''

    # Create sample Tags
    tag_list = []
    for tmp in ["Electronics", "Batteries", "Bag", "Package", "Connectors"]:
        tag = Tag()
        tag.name = tmp
        tag_list.append(tag)
        db.session.add(tag)

    # Create sample products
    sample_text = [
        {
            'title': "de Finibus Bonorum et Malorum - Part I",
            'content': "Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor \
                        incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud \
                        exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure \
                        dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. \
                        Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt \
                        mollit anim id est laborum."
        },
        {
            'title': "de Finibus Bonorum et Malorum - Part II",
            'content': "Sed ut perspiciatis unde omnis iste natus error sit voluptatem accusantium doloremque \
                        laudantium, totam rem aperiam, eaque ipsa quae ab illo inventore veritatis et quasi architecto \
                        beatae vitae dicta sunt explicabo. Nemo enim ipsam voluptatem quia voluptas sit aspernatur \
                        aut odit aut fugit, sed quia consequuntur magni dolores eos qui ratione voluptatem sequi \
                        nesciunt. Neque porro quisquam est, qui dolorem ipsum quia dolor sit amet, consectetur, \
                        adipisci velit, sed quia non numquam eius modi tempora incidunt ut labore et dolore magnam \
                        aliquam quaerat voluptatem. Ut enim ad minima veniam, quis nostrum exercitationem ullam \
                        corporis suscipit laboriosam, nisi ut aliquid ex ea commodi consequatur? Quis autem vel eum \
                        iure reprehenderit qui in ea voluptate velit esse quam nihil molestiae consequatur, vel illum \
                        qui dolorem eum fugiat quo voluptas nulla pariatur?"
        },
        {
            'title': "de Finibus Bonorum et Malorum - Part III",
            'content': "At vero eos et accusamus et iusto odio dignissimos ducimus qui blanditiis praesentium \
                        voluptatum deleniti atque corrupti quos dolores et quas molestias excepturi sint occaecati \
                        cupiditate non provident, similique sunt in culpa qui officia deserunt mollitia animi, id \
                        est laborum et dolorum fuga. Et harum quidem rerum facilis est et expedita distinctio. Nam \
                        libero tempore, cum soluta nobis est eligendi optio cumque nihil impedit quo minus id quod \
                        maxime placeat facere possimus, omnis voluptas assumenda est, omnis dolor repellendus. \
                        Temporibus autem quibusdam et aut officiis debitis aut rerum necessitatibus saepe eveniet \
                        ut et voluptates repudiandae sint et molestiae non recusandae. Itaque earum rerum hic tenetur \
                        a sapiente delectus, ut aut reiciendis voluptatibus maiores alias consequatur aut perferendis \
                        doloribus asperiores repellat."
        }
    ]


    user_list = User.query.all()
    for i in range(1):
        entry = random.choice(sample_text)  # select text at random
        prod = Product()
        prod.user = user_list[0]
        prod.title = entry['title']
        prod.text = entry['content']
        prod.preview_filename = "test"
        tmp = int(1000 * random.random())  # random number between 0 and 1000:
        prod.date = datetime.datetime.now() - datetime.timedelta(days=tmp)
        prod.tags = random.sample(tag_list, 2)  # select a couple of tags at random
        db.session.add(prod)
    
    '''
    # Create a sample Tree structure
    trunk = Tree(name="Trunk")
    db.session.add(trunk)
    for i in range(5):
        branch = Tree()
        branch.name = "Branch " + str(i+1)
        branch.parent = trunk
        db.session.add(branch)
        for j in range(5):
            leaf = Tree()
            leaf.name = "Leaf " + str(j+1)
            leaf.parent = branch
            db.session.add(leaf)
    '''
    db.session.commit()
    return

def BuildAdmin(app):
    db.app = app
    db.init_app(app)
    # Create admin
    admin = fadmin.Admin(app, name='CypressBoat Admin', template_mode='bootstrap3')
    # Add views
    admin.add_view(UserAdmin(User, db.session))
    admin.add_view(TagsAdmin(db.session))
    admin.add_view(ProductAdmin(model=Product, session=db.session, endpoint="productadmin"))
    admin.add_view(ImageAdmin(model=Image, session=db.session, endpoint="imageadmin"))
    admin.add_view(SearchView(template='Search.html', name="Search", url='/search', endpoint='search'))
    #admin.add_view(TreeView(Tree, db.session))

def BuildEmptyDB(app):
    ''' Creates an empty database file and a repository to handle database upgrades '''
    db.app = app        
    db.init_app(app)
    # Build a sample db on the fly, if one does not exist yet.
    if not os.path.exists(app.config['DATABASE_FILE']):
        build_sample_db()
    '''    
    if not os.path.exists(app.config['SQLALCHEMY_MIGRATE_REPO']):
        api.create(app.config['SQLALCHEMY_MIGRATE_REPO'], 'database repository')
        api.version_control(app.config['SQLALCHEMY_DATABASE_URI'], app.config['SQLALCHEMY_MIGRATE_REPO'])
    else:
        api.version_control(app.config['SQLALCHEMY_DATABASE_URI'], app.config['SQLALCHEMY_MIGRATE_REPO'], api.version(app.config['SQLALCHEMY_MIGRATE_REPO']))
    '''
    # Start app
    #app.run(debug=True)
