from flask_wtf import Form
from flask_wtf.file import FileField, FileAllowed
from wtforms import StringField, RadioField, BooleanField, TextAreaField, DateField, TextField
from wtforms.validators import DataRequired, Length, InputRequired
from flask_security.forms import RegisterForm, ConfirmRegisterForm, LoginForm

from CypressBoat import app
from CypressBoat.Admin import db, User



#----------------------------------------------------------------
# User Admin Forms


class EditUserForm(Form):
    ''' Allow the user to edit their personal details '''
    username = StringField('username', validators=[DataRequired()])
    email = StringField('email', validators=[DataRequired()])
    first_name = StringField('first_name', validators=[DataRequired()])
    last_name = StringField('last_name', validators=[DataRequired()])
    about_me = TextAreaField('about_me', validators=[Length(min=0, max=140)])

class ChangePasswordForm(Form):
    ''' Allow the user to update their password '''
    old_password = StringField('old_password', validators=[DataRequired()])
    new_password1 = StringField('new_password1', validators=[DataRequired()])
    new_password2 = StringField('new_password2', validators=[DataRequired()])



#----------------------------------------------------------------
# Flask Securty Forms

class ExtendedRegisterForm(RegisterForm):
    username = StringField('Username', validators=[DataRequired()])

class ExtendedConfirmRegisterForm(ConfirmRegisterForm):
    username = StringField('Username', validators=[DataRequired()])

class ExtendedLoginForm(LoginForm):
    email = StringField('Username', [InputRequired()])

#----------------------------------------------------------------
# Post & Collection

class AddCommentForm(Form):
    title = StringField('title', validators=[DataRequired()])
    body = StringField('body', validators=[DataRequired()])

class UploadForm(Form):
    date = DateField('Name', validators =...)
    image = FileField(validators=[DataRequired()])


class ContactForm(Form):
    name = TextField("Name")
    email = TextField("Email", validators=[DataRequired()])
    subject = TextField("Subject")
    message = TextAreaField("Message", validators=[DataRequired()])
    attachment = FileField(validators=[FileAllowed(['doc','docx','pdf'],"Word (doc, docx) or PDF documents only please")])

