#----------------------------------------------------------------------------------
# (c) Christian Lynch 2016
#----------------------------------------------------------------------------------

# flask/database related imports
from flask import Flask, request, session, g, redirect, url_for, abort, \
     render_template, render_template_string, flash
from flask_mail import Mail, Message
from flask_security import Security, current_user, roles_required, roles_accepted, login_required
from sqlalchemy import and_, or_
import datetime
from flask_admin import expose
from werkzeug import secure_filename
from werkzeug.datastructures import FileStorage

# import other parts of the web application
from CypressBoat.Forms import EditUserForm, ChangePasswordForm, AddCommentForm, ContactForm
from CypressBoat import app
from CypressBoat.Admin import db, User
import CypressBoat.Admin as DOA

mail = Mail(app)

# Google Verification control file
#------------------------------------------------------------------------------------
@app.route('/google6f5533410d91160c.html')
def GoogleVerification():
    return render_template('google6f5533410d91160c.html')

# to make google happy we give a robots.txt to confirm we DO want the site to be indexed
# dont know why google likes this because supposedly no robots meaks the site can be crawled?!
@app.route('/robots.txt')
def Robots():
    return render_template('robots.txt')

#-----------------------------------------------------------------------------------
# User settings

@app.route('/admin/profile', methods=['GET', 'POST'])
@login_required
def EditUser():
    form = EditUserForm(obj=current_user)
    if current_user.upload_quota is None:
        current_user.upload_quota = app.config['USER_DEFAULT_UPLOAD_QUOTA']
    if current_user.upload_quota_used is None:
        current_user.upload_quota_used = 0
    MBUsed = "%.2f" % (float(current_user.upload_quota_used)/float(1000000))
    MBQuota = "%.2f" % (float(current_user.upload_quota)/float(1000000))
    if form.validate_on_submit():
        current_user.username = form.username.data
        current_user.first_name = form.first_name.data
        current_user.last_name = form.last_name.data
        current_user.about_me = form.about_me.data        
        current_user.email = form.email.data
        db.session.commit()
        flash('Your changes have been saved.')
        return redirect('/admin')
    return render_template('EditUser.html', form=form, MBUsed=MBUsed, MBQuota=MBQuota)

@app.route('/admin/change_password', methods=['GET', 'POST'])
@login_required
def ChangePassword():
    form = ChangePasswordForm()
    if form.validate_on_submit():
        old_password = form.old_password.data
        new_password1 = form.new_password.data
        new_password2 = form.new_password.data
        if old_password != current_user.password:
            flash("Your old password doesn't match.")
            return redirect('/admin/profile')
        if old_password != current_user.password:
            flash("Your new passord should be exactly repeated.")
            return redirect('/admin/profile')
        current_user.password = new_password1
        db.session.commit()
        flash('Your password has been updated successfully.')
        return redirect('/admin/profile')
    return render_template('ChangePassword.html', form=form)


#-----------------------------------------------------------------
# Comment functionality 

@app.route('/product/add_comment', methods=['GET', 'POST'])
@login_required
def AddProductComment():
    product_id = request.args.get('id')
    form = AddCommentForm()
    if product_id is None:
        page_not_found()
    entries = DOA.Product.query.filter(DOA.Product.id==product_id).all()
    if len(entries) != 1:
        page_not_found()
    product=entries[0]

    if form.validate_on_submit():
        com = DOA.ProductComment()
        com.title = form.title.data
        com.body = form.body.data
        com.user = current_user
        com.product = product
        com.timestamp = datetime.datetime.now() 
        db.session.add(com)
        db.session.commit()
        flash('Your comment has been posted successfully.')
        return redirect('/product?id='+str(product_id))
    return render_template('AddComment.html', form=form, Title=product.title)


#--------------------------------------------------------------------------
# first some routing rules

@app.errorhandler(404)
def page_not_found(e=None):
    # your processing here
    return render_template('/content/other/404.html')

# routing rule for any pages that can be accessed directly by the user through standard html
@app.route('/static/<obj>')
def Static(obj):
    return app.send_static_file("/static/" + obj)

# routing rule for any images that can be accessed directly by the user through standard html
@app.route('/images/<obj>')
def Images(obj):
    return app.send_static_file("/static/images/" + obj)


#--------------------------------------------------------------------------
# Main content views 

@app.route('/info/<subpage>')
def About(subpage):
    return render_template('/info/'+ subpage + ".html")


@app.route('/contact', methods=['GET', 'POST'])
def contact():
    form = ContactForm()
    if request.method == 'POST':
        if form.validate_on_submit():
            msg = Message("Website message: %s" % (form.subject.data),
                          sender='info@cypressboat.co.uk',
                          recipients=['info@cypressboat.co.uk'])
            msg.body = """
            From: %s <%s>,
            %s
            """ % (form.name.data, form.email.data, form.message.data)
            if isinstance(form.attachment.data, FileStorage):
                fn = secure_filename(form.attachment.data.filename)
                if form.attachment.data.filename != "":
                    msg.attach(fn, 'application/octet-stream', form.attachment.data.read())
            mail.send(msg)
            flash('Your message has been sent.')
            return render_template('Home.html', form=form)
    return render_template('Contact.html', form=form)

@app.route('/productlist')
def ProrductList():
    return render_template('ProductList.html')

def ContentBlockInc(path):
    rstr = "{% extends \"layout.html\" %}{% block content %}{% include \'" + path +"\' %}{% endblock %}"
    return rstr

def ContentBlock(content):
    rstr = "{% extends \"layout.html\" %}{% block content %}" + content +"{% endblock %}"
    return rstr

# anything in the content folder is lauched directly, with enclosing content block definitions
@app.route('/content/<path:path>')
def Content(path, error=None):
    return render_template_string(ContentBlockInc("/content/" + path), error=error)



def GenerateTagsHTMLRow(RequiredTags, OptionalTags):
    ''' Returns html representing a horizontal row of tags '''
    txt = ""
    TType = "products"
    for t in RequiredTags:
        txt = txt + "<a href=\""+ TType + "?requiredtags=" + t + "\" class=\"RequiredTag\">"+ t + "</a>"
    for t in OptionalTags: 
        txt = txt + "<a href=\""+ TType + "?requiredtags=" + t + "\" class=\"OptionalTag\">"+ t + "</a>"
    return(txt)


def GetMatchingCollections(RequiredTags, OptionalTags):
    ''' This function returns a list of collections that have certain tags:
    all of the required tags need to be present, and at least one match of the Optional Tags (if any) need to match '''
    #return []    
    
    if len(RequiredTags) == 0 and len(OptionalTags) == 0:	 
        entries = DOA.Collection.query.all() 
    else:
        #entries=DOA.Collection.query.filter(and_(DOA.Collection.tags.any(name="Power"),DOA.Collection.tags.any(name="China"))).all()
        OrArgList = [DOA.Collection.tags.any(name=ot) for ot in OptionalTags]
        AndArgList = [DOA.Collection.tags.any(name=ot) for ot in RequiredTags]
        entries = DOA.Collection.query.filter(and_(or_(*OrArgList),and_(*AndArgList))).all()
    return entries


def GetMatchingItems(ItemObject, RequiredTags, OptionalTags, page_number, page_items):
    ''' This function returns a list of items that have certain tags:
    all of the required tags need to be present, and at least one match of the Optional Tags (if any) need to match '''
    if RequiredTags is None and OptionalTags is None:	 
        entries = ItemObject.query.paginate(page_number, page_items, False)
    else:
        #entries=DOA.Collection.query.filter(and_(DOA.Collection.tags.any(name="Power"),DOA.Collection.tags.any(name="China"))).all()
        if OptionalTags is not None:
            OrArgList = [ItemObject.tags.any(name=ot) for ot in OptionalTags]
        else: OrArgList = []
        if RequiredTags is not None:
            AndArgList = [ItemObject.tags.any(name=ot) for ot in RequiredTags]
        else: AndArgList = []
        entries = ItemObject.query.filter(and_(or_(*OrArgList),and_(*AndArgList))).paginate(page_number, page_items, False)
    return entries


def BuildQueryString(optional_tags, required_tags, page_number, page_items=None, sort_by=None):
    ''' Given a set of required and optional tags, this function creates a query string that
    encodes them '''
    # first replace spaces with plusses in the tags
    required_tags = [s.replace(" ","+") for s in required_tags]
    optional_tags = [s.replace(" ","+") for s in optional_tags]

    # then join the tags together using a full-stop as a separator
    if required_tags is not None and len(required_tags) > 0:
        required_tags = ".".join([d for d in required_tags])
    else: required_tags = None
    if optional_tags is not None and len(optional_tags) > 0:
        optional_tags = ".".join([d for d in optional_tags])
    else: optional_tags = None

    # then build the final string putting all the pieces together
    if required_tags or optional_tags: QueryString = "?"
    else: QueryString = ""
    QueryItems = []
    if required_tags: QueryItems.append("requiredtags=" + required_tags)
    if optional_tags: QueryItems.append("optionaltags=" + optional_tags)
    if page_number is not None: QueryItems.append("page=" + str(page_number))
    if page_items is not None and page_items != app.config["DO_PAGINATE_ITEMS"]: QueryItems.append("items=" + str(page_items))
    if sort_by is not None: QueryItems.append("sort_by=" + sort_by)
    QueryString = "?" + "&".join(QueryItems)
    return(QueryString)
 



def SearchQueryResults(ModelObject, TemplateName, Title):
    ''' Renders a page showing a list of search results matching various criteria.'''
    optional_tags_arg = request.args.get('optionaltags')
    required_tags_arg = request.args.get('requiredtags')
    page_number = request.args.get('page')
    if page_number is not None: page_number=int(page_number)
    page_items = request.args.get('items')
    if page_items is not None: page_items=int(page_items)
    sort_by = request.args.get('sort_by')
    if page_number is None: page_number = 1
    else: page_number = int(page_number)
    if page_items is None: page_items = app.config["DO_PAGINATE_ITEMS"]
    else: page_items = int(page_items)

    # tags are split in the query string 
    if optional_tags_arg is None: optional_tags=[]
    else: optional_tags=optional_tags_arg.split('.')
    if required_tags_arg is None: required_tags=[]
    else: required_tags=required_tags_arg.split('.')
    
    # spaces are encoded as "+"
    required_tags = [s.replace("+"," ") for s in required_tags]
    optional_tags = [s.replace("+"," ") for s in optional_tags]

    posts = GetMatchingItems(ModelObject, required_tags, optional_tags, page_number, page_items)
    FirstItem = 1 + page_items * (page_number - 1)
    LastItem = FirstItem + len(posts.items) - 1
    if posts.total == 0:
        ResultsStr = "No matching results"
    else: ResultsStr = "Showing items %i-%i of %i" % (FirstItem, LastItem, posts.total)

    NextPageURL = BuildQueryString(optional_tags, required_tags, page_number - 1, page_items)
    PreviousPageURL = BuildQueryString(optional_tags, required_tags, page_number + 1, page_items)
    ProductData = []
    for p in posts.items:
        if p.summary_image is not None:
            sum_im = "/static/uploadedfiles/images/"+str(p.summary_image.id) + ".jpg"
        else: sum_im = "/static/images/NoProductImage.svg"
        ProductData.append((p, sum_im, p.GetTagStringList()))
    return render_template(TemplateName, Title=Title, posts=posts, productdata=ProductData, required_tags=required_tags, \
        optional_tags=optional_tags, NextPageURL=NextPageURL, PreviousPageURL=PreviousPageURL, ResultsStr=ResultsStr)

@app.route('/')
def Home():
    return(render_template("Home.html"))


#---------------------------------------------------------------------------------
# User profile page

@app.route('/user/<username>')
@login_required
def user(username):
    user = User.query.filter_by(username=username).first()
    if user == None:
        flash('User %s not found.' % username)
        return redirect(url_for('index'))
    posts = [
        {'author': user, 'body': 'Test post #1'},
        {'author': user, 'body': 'Test post #2'}
    ]
    return render_template('UserProfile.html',
                           user=user,
                           posts=posts)

# -----------------------------------------------------------------------
# These functions can only be accessed with developer login credentials


@app.route('/developer')
@roles_required('admin')
def Developer():
    return render_template_string(ContentBlockInc("developer.html"))

@app.route('/rebuild-database')
@roles_required('admin')
def RebuildDatabase():
    DOA.BuildEmptyDB(app)
    flash('Database has been rebuilt')
    return redirect(url_for('Home'))

@app.route('/backup-database')
@roles_required('admin')
def BackupDatabase():
    #DOA.BuildEmptyDB(app)
    flash('Database has been backed-up')
    return redirect(url_for('Home'))


#-------------------------------------------------------------------------------------
# Search


